"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (Object.prototype.hasOwnProperty.call(b, p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Dam = void 0;
var computedFrom_1 = require("../computedFrom");
var Event_1 = require("../polyfill/Event");
var Shell_1 = require("../Shell");
/**
 * The Dam simulate a Websocket Opening and Closing
 * Example of use include simulating a connection opening after an authentication
 *
 * @example
 * ```typescript
 *
 * const dam = new Dam(ws);
 * expect(dam.send("test")).to.throw;
 * dam.status = Dam.OPEN;
 * \// the open event is also dispatched
 * expect(dam.send("test")).to.not.throw;
 * ```
 */
var Dam = /** @class */ (function (_super) {
    __extends(Dam, _super);
    /**
     * The created Dam will be *CLOSED*
     * @param ws  An object compatible with the WebSocket interface.
     */
    function Dam(ws) {
        var _this = _super.call(this) || this;
        _this.openSent = false;
        _this.buffer = [];
        _this._status = "CLOSED";
        _this._readyState = _this.CLOSED;
        _this.ws = ws;
        _this.forwardEvents();
        return _this;
    }
    Object.defineProperty(Dam.prototype, "status", {
        get: function () {
            return this._status;
        },
        set: function (value) {
            if (this._status === value) {
                return;
            }
            this._status = value;
            if (this._status === Dam.OPEN) {
                if (this.ws.readyState === this.ws.OPEN) {
                    this.dispatchEvent(new Event_1.default("open"));
                }
                this.flush();
            }
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(Dam.prototype, "readyState", {
        /**
         * Compatible with the WebSocket readyState, combining the _status_ of the _Dam_ and the ready state
         * of the underlying WebSocket
         * @returns {number} WebSocket constants (CONNECTING, OPEN, CLOSING, CLOSED)
         */
        get: function () {
            if (this._status === Dam.CLOSED &&
                (this.ws.readyState === this.ws.CONNECTING || this.ws.readyState === this.ws.OPEN)) {
                return this.CONNECTING;
            }
            else {
                return this.ws.readyState;
            }
        },
        enumerable: false,
        configurable: true
    });
    /**
     * Dispatch an event, some event are delayed if the Dam is CLOSED
     * @param {} The Event object to be dispatched.
     * @returns {boolean} The return value is **false** if event is cancelable and at least one of the event
     * handlers which handled this event called Event.preventDefault(). Otherwise it returns true.
     */
    Dam.prototype.dispatchEvent = function (evt) {
        if (evt.type === "close") {
            this.dispatchCloseEvent(evt);
        }
        else if (evt.type === "open") {
            this.dispatchOpenEvent(evt);
        }
        else {
            this.dispatchOrBuffer(evt);
        }
        return true;
    };
    /**
     * Same as the send method of the WebSocket except that this will throw an error if the Dam is closed
     * @param data The data to send to the server. It may be one of the following types:
     * - **USVString**
     * A text string. The string is added to the buffer in UTF-8 format, and the value of bufferedAmount is increased
     * by the number of bytes required to represent the UTF-8 string.
     * - **ArrayBuffer**
     * You can send the underlying binary data used by a typed array object; its binary data contents are queued in
     * the buffer, increasing the value of bufferedAmount by the requisite number of bytes.
     * - **Blob**
     * Specifying a Blob enqueues the blob's raw data to be transmitted in a binary frame. The value of bufferedAmount
     * is increased by the byte size of that raw data.
     * - **ArrayBufferView**
     * You can send any JavaScript typed array object as a binary frame; its binary data contents are queued in the
     * buffer, increasing the value of bufferedAmount by the requisite number of bytes.
     */
    Dam.prototype.send = function (data) {
        if (this._status === Dam.OPEN) {
            _super.prototype.send.call(this, data);
        }
        else {
            throw new Error("WebSocket is closed");
        }
    };
    Dam.prototype.dispatchCloseEvent = function (evt) {
        _super.prototype.dispatchEvent.call(this, evt);
        this.openSent = false;
    };
    Dam.prototype.dispatchOpenEvent = function (evt) {
        if (this._status === Dam.OPEN) {
            _super.prototype.dispatchEvent.call(this, evt);
            this.openSent = true;
        }
    };
    Dam.prototype.dispatchOrBuffer = function (evt) {
        if (this._status === Dam.OPEN) {
            _super.prototype.dispatchEvent.call(this, evt);
        }
        else {
            this.buffer.unshift(evt);
        }
    };
    Dam.prototype.flush = function () {
        var evt;
        while (evt = this.buffer.pop()) { // tslint:disable-line:no-conditional-assignment
            _super.prototype.dispatchEvent.call(this, evt);
        }
    };
    /**  The dam is open and will let messages pass through */
    Dam.OPEN = "OPEN";
    /**  The dam is closed and will stop the messages */
    Dam.CLOSED = "CLOSED";
    __decorate([
        computedFrom_1.computedFrom("_status")
    ], Dam.prototype, "status", null);
    __decorate([
        computedFrom_1.computedFrom("_status", "ws.readyState")
    ], Dam.prototype, "readyState", null);
    return Dam;
}(Shell_1.Shell));
exports.Dam = Dam;
//# sourceMappingURL=Dam.js.map