"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (Object.prototype.hasOwnProperty.call(b, p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
exports.Pipe = void 0;
var CloseEvent_1 = require("../polyfill/CloseEvent");
var MessageEvent_1 = require("../polyfill/MessageEvent");
var Shell_1 = require("../Shell");
/**
 * The Pipe create a simple multiplexing for string messages
 *
 * @example
 * ```typescript
 *
 * Client 1
 * const pipeA = new Pipe(ws, "A");
 * pipeA.send("message sent on pipe A")
 * const pipeB = new Pipe(ws, "B");
 *
 * Client 2
 * const pipeA = new Pipe(ws, "A");
 * pipeA.on("message", (str)=> console.log(`received ${str} on pipeA`)
 * const pipeB = new Pipe(ws, "B");
 * pipeB.on("message", (str)=> console.log(`received ${str} on pipeB`)
 * ```
 */
var Pipe = /** @class */ (function (_super) {
    __extends(Pipe, _super);
    /**
     * @param {} ws
     * @param {string} a string used to mark the messages channel, transparent for the user
     * @param {number} the maximum prefix length, should be as short as possible and
     *                 identical for all pipes on a WebSocket, default to 4
     */
    function Pipe(ws, channel, prefixLength) {
        if (prefixLength === void 0) { prefixLength = 4; }
        var _this = _super.call(this) || this;
        _this.prefixLength = prefixLength;
        _this.pipeReadyState = null;
        if (typeof (channel) !== "string" || !channel.length || channel.length > _this.prefixLength) {
            throw new Error("Channel should be a string between 1 and " + _this.prefixLength + " characters");
        }
        _this.channel = ("" + Pipe.repeatString(" ", _this.prefixLength) + channel).slice(_this.prefixLength * -1);
        _this.ws = ws;
        _this.forwardEvents();
        return _this;
    }
    Pipe.repeatString = function (str, count) {
        var res = "";
        for (var i = count; i > 0; i--) {
            res += str;
        }
        return res;
    };
    /**
     * Closes the WebSocket connection or connection attempt, if any.
     * If the connection is already CLOSED, this method does nothing.
     * @param {number} A numeric value indicating the status code explaining why the connection is being closed.
     * If this parameter is not specified, a default value of 1005 is assumed.
     * See the list of status codes https://developer.mozilla.org/en-US/docs/Web/API/CloseEvent#Status_codes
     * on the CloseEvent page for permitted values.
     * @param {string} A human-readable string explaining why the connection is closing.
     * This string must be no longer than 123 bytes of UTF-8 text (not characters).
     */
    Pipe.prototype.close = function (code, reason) {
        var _this = this;
        if (code === void 0) { code = 1000; }
        if (this.pipeReadyState) {
            return;
        }
        this.stopForwardingEvents();
        this.pipeReadyState = this.CLOSING;
        setTimeout(function () {
            _this.pipeReadyState = _this.CLOSED;
            _this.dispatchEvent(new CloseEvent_1.default("close", { code: code, reason: reason, wasClean: true }));
        }, 0);
    };
    /**
     * Send a message through the pipe, unlike regular _send_ method, a pipe only accept string messages
     * @param {string} data
     */
    Pipe.prototype.send = function (data) {
        if (this.pipeReadyState) {
            return;
        }
        if (typeof (data) !== "string") {
            throw new Error("Pipe only support sending string, you passed a type " + typeof (data));
        }
        _super.prototype.send.call(this, this.channel + data);
    };
    /**
     * Dispatch an event to this EventTarget.
     * @param {} The Event object to be dispatched.
     * @returns {boolean} The return value is **false** if event is cancelable and at least one of the event
     * handlers which handled this event called Event.preventDefault(). Otherwise it returns true.
     */
    Pipe.prototype.dispatchEvent = function (evt) {
        if (evt.type === "message") {
            var e = evt;
            if (typeof (e.data) === "string" && this.channel === e.data.substr(0, this.prefixLength)) {
                _super.prototype.dispatchEvent.call(this, new MessageEvent_1.default("message", {
                    data: e.data.substr(this.prefixLength),
                    origin: e.origin,
                    ports: e.ports,
                    source: e.source,
                }));
                return false;
            }
        }
        else {
            _super.prototype.dispatchEvent.call(this, evt);
        }
    };
    Pipe.prototype.getReadyState = function () {
        return this.pipeReadyState || _super.prototype.getReadyState.call(this);
    };
    return Pipe;
}(Shell_1.Shell));
exports.Pipe = Pipe;
//# sourceMappingURL=Pipe.js.map